
package io.github.bucket4j.grid.jcache.infinispan;

import io.github.bucket4j.Bandwidth;
import io.github.bucket4j.Bucket4j;
import io.github.bucket4j.grid.GridBucketState;
import io.github.bucket4j.grid.RecoveryStrategy;
import io.github.bucket4j.grid.jcache.JCache;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import javax.cache.Cache;
import javax.cache.CacheManager;
import javax.cache.Caching;
import javax.cache.spi.CachingProvider;
import java.net.URI;
import java.net.URISyntaxException;
import java.time.Duration;

public class InfinispanJCacheTest {

    static URI configurationUri = null;

    static {
        try {
            configurationUri = InfinispanJCacheTest.class.getResource("/infinispan-jcache-cluster.xml").toURI();
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private static Cache<String, GridBucketState> cache;
    private static CacheManager cacheManager;

    @BeforeClass
    public static void setup() {
        ClassLoader tccl = InfinispanJCacheTest.class.getClassLoader();
        CachingProvider cachingProvider = Caching.getCachingProvider("org.infinispan.jcache.embedded.JCachingProvider");
        cacheManager = cachingProvider.getCacheManager(configurationUri, new TestClassLoader(tccl));
        cache = cacheManager.getCache("my_buckets");

    }

    @AfterClass
    public static void shutdown() {
        if (cacheManager != null) {
            cacheManager.close();
        }
    }

    @Test(expected = UnsupportedOperationException.class)
    public void checkThatInfinispanProviderUnsupported() {
        Bucket4j.extension(JCache.class).builder().
                addLimit(Bandwidth.simple(10, Duration.ofSeconds(60))).
                build(cache, "42", RecoveryStrategy.RECONSTRUCT);
    }

    public static class TestClassLoader extends ClassLoader {
        public TestClassLoader(ClassLoader parent) {
            super(parent);
        }
    }

}
